' --------------------------------------------------------------------------------------
'
' GrandNapPat.VB                                                  Ulrich Mller 24.09.02
'                                                                 PCS1700   VB.NET Final
' Grand Napoleon Patience : Logik, Legen und Anzeigen
' Basis PotiPat.DLL und QCard32.DLL
'
' Ablage  : obere Reihe mit den Pltzen 0 - 7 und Mittellcke in Breite von 3 Pltzen
' Auslage : mittlerer Bereich mit 10 Leitern (0 - 9 ) mit 40 max. maxCard Karten
' Keller  : hier nicht bentigt
' Talon   : unsichtbar, ber linker, oberer Ecke
'
' --------------------------------------------------------------------------------------

Imports PotiPat

Public Class NapException
  Inherits Exception
  Public Sub New(ByVal message$)
    MyBase.New(message)
  End Sub
End Class

' GrandNapPatience -----------------------------------------------------------------------
'             Spielspezifische Methoden fr die Patience Grand Napoleon
' -----------------------------------------------------------------------------------
Public Class GrandNapPatience
  Inherits PotiPatience

  Public Event GameWon(ByVal Pati As GrandNapPatience)

  ' --------------------------- Private Felder ---------------------------------------

  Private Talon As KartenPaket   ' --- KartenPaket
  Private Ablage As Ablage48     ' --- Ablage
  Private AusLage As AuslageV    ' --- Karten der Auslage

  Private SrcCard As Card, SrcPos As New CardPos()
  Private CardList(15), anzCard As Integer

  Private pnlDisplay As Panel, lblRunde, lblRestKarten As Label
  Private SpielRunde As Integer, BeimAuslegen As Boolean

  ' New ------------------------------------------------------------------------------
  '     Konstruktor : Aufsetzen QCard32Pool, Bestimmen der Spielbereichskoordinaten
  '                   Instanzieren der Spielbereiche
  ' ----------------------------------------------------------------------------------
  Sub New(ByVal Canvas As Form, ByVal pnlDisplay As Panel, _
                                ByVal lblRunde As Label, ByVal lblRestKarten As Label)
    MyBase.New(Canvas)

    Dim XNull, XTalon, YNull, YAblage As Integer

    Me.lblRunde = lblRunde
    Me.lblRestKarten = lblRestKarten
    Me.pnlDisplay = pnlDisplay

    ' --- Spielbereichs Koordinaten ------------------------
    XNull = 16
    YAblage = 16
    YNull = YAblage + qc.CardHeight + 32
    XTalon = XNull + qc.CardWidthS * 9

    ' --- Spielbereichs Objekte -----
    Talon = New KartenPaket(qc, 2, -qc.CardWidthS, -qc.CardHeightS)
    Ablage = New Ablage48(qc, 8, qc.CardWidthS, XNull, YAblage, 3)
    AusLage = New AuslageV(qc, 11, qc.CardWidthS, XNull, YNull, Canvas.Height - YNull)

    ' --- Anzeige-Labels Ausrichten ----
    
    pnlDisplay.Top = Ablage.Top + 5
    pnlDisplay.Left = Ablage.Left + Ablage.Range.Width / 2 - pnlDisplay.Width / 2

  End Sub

  '
  ' --------- Private Methoden ----------------------------------------------------
  '

  ' FindDestPos ------------------------------------------------------------------------
  '             Bestimmen der angegebenen Position (in Pixelkoordinaten)
  ' ------------------------------------------------------------------------------------
  Private Function FindDestPos(ByVal X%, ByVal Y%) As CardPos
    Dim DestPos As New CardPos()

    DestPos.CardNr = 0
    If Ablage.Range.Contains(X, Y) Then
      DestPos.Area = AreaCode.Ablage
      DestPos.PosC = Ablage.XToCol(X)
    ElseIf AusLage.Range.Contains(X, Y) Then
      DestPos.Area = AreaCode.Auslage
      DestPos.PosC = AusLage.XToCol(X)
    Else
      DestPos.Area = AreaCode.none
      DestPos.PosC = 0
    End If
    DestPos.PosR = -1                            ' --- Keine Zeilenangabe
    Return DestPos
  End Function

  '
  ' ------ Public Methoden -------------------------------------------------------
  '

  ' SpielStarten -------------------------------------------------------------------
  '              Starten eines neuen Spieles : Initwerte, Mischen, Auslegen 
  ' --------------------------------------------------------------------------------
  Public Overrides Sub SpielStarten(ByVal SpielNr%)

    Canvas.CreateGraphics.Clear(Color.Green)
    Talon.Shuffle(SpielNr)
    Ablage.Clear()
    AusLage.Clear()
    SpielRunde = 0
    NachsteRunde()
  End Sub

  ' Refresh -------------------------------------------------------------------------
  '         Wiederherstellen des aktuellen Kartenbildes 
  ' ---------------------------------------------------------------------------------
  Public Overrides Sub Refresh()
    Ablage.Paint()
    AusLage.Paint()
  End Sub

  ' StartZiehen ---------------------------------------------------------------------
  '             Starten des Ziehens einer Karte wenn Maus ber Karte, sonst merken
  '             der Mausposition
  ' ---------------------------------------------------------------------------------
  Public Overrides Sub StartZiehen(ByVal X%, ByVal Y%)
    Dim SrcNr% = qc.InitDrag(X, Y)   ' --- Starten Drag, wenn Maus auf Karte
    Dim i%, j%
    If SrcNr = 0 Then
      SrcPos.CardNr = 0
      SrcPos.PicX = X
      SrcPos.PicY = Y
      qc.AbortDrag()                 ' --- Keine Karte, Merken Mauspositione fr 
      DragMode = DragCode.None       '     DoppelClick, Abbruch Drag

    Else                             ' --- Auf Karte
      If qc.GetCardBlocked(SrcNr) Then
        DragMode = PotiPatience.DragCode.None
        qc.AbortDrag()
      Else
        DragMode = DragCode.One
        Select Case qc.GetUser2(SrcNr)  ' --- Bestimmen der Karten-Area
          Case AreaCode.Ablage          ' --- Lesen der obersten Karte der jeweiligen Area
            SrcCard = Ablage.TopMostCard(qc.GetUser3(SrcNr))
          Case AreaCode.Auslage
            SrcCard = AusLage.TopMostCard(qc.GetUser3(SrcNr))
          Case AreaCode.Keller
          Case AreaCode.Talon
        End Select
      End If
    End If
  End Sub

  ' WeiterZiehen ------------------------------------------------------------------
  '              Fortsetzen des Ziehens
  ' -------------------------------------------------------------------------------
  Public Overrides Sub WeiterZiehen(ByVal X%, ByVal Y%)
    Select Case DragMode
      Case DragCode.Block
        Return
      Case DragCode.One
        qc.DoDrag(X, Y)
      Case Else
    End Select
  End Sub

  ' EndeZiehen ---------------------------------------------------------------------
  '            Beenden des Ziehens, Bestimmen der Zielposition, Ablegen der Karte
  '            bzw. Rcklegen einer unpassenden Karte
  ' --------------------------------------------------------------------------------
  Public Overrides Sub EndeZiehen(ByVal X%, ByVal Y%)

    Dim DestPos As CardPos
    Select Case DragMode

      ' ------------------ SingleDrag ----------------------

    Case DragCode.One
        qc.EndDrag(X, Y)
        DestPos = FindDestPos(X, Y)             ' --- Zielposition
        Select Case DestPos.Area                ' --- Prfen des Platzes der Area
          Case AreaCode.Ablage                  '     Ablage
            If (DestPos.PosC < 4 And Ablage.IsAscendingSuit(SrcCard, DestPos.PosC)) OrElse _
                DestPos.PosC > 3 And Ablage.IsDescendingSuit(SrcCard, DestPos.PosC) Then
              Ablage.AddCard(AusLage.RemoveCard(SrcCard.PosC), DestPos.PosC)
            Else
              qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY)
            End If
          Case AreaCode.Auslage                 ' --- Auslage
            If Not BeimAuslegen And AusLage.IsUpDownSuit(SrcCard, DestPos.PosC) Then
              AusLage.AddCard(AusLage.RemoveCard(SrcCard.PosC), DestPos.PosC)
            Else
              qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY)
            End If
          Case Else
            qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY)
        End Select

        ' ---------------------- BlockDrag innerhalb der Auslage --------------------

      Case DragCode.Block

        ' --------------------- Else ------------------------------------------------

      Case Else
        Return
    End Select

    ' --- Beenden Drag -------------------------------------------------

    DragMode = DragCode.None
    lblRestKarten.Text = 104 - Ablage.CardCount
    If Ablage.CardCount = 104 Then RaiseEvent GameWon(Me)

  End Sub

  ' KarteAblegen -------------------------------------------------------------------
  '              Prfen und angegebene Karte ablegen (nach z.B. DoppelClick)
  ' --------------------------------------------------------------------------------
  Public Overrides Sub KarteAblegen()
    Return
  End Sub

  ' NachsteRunde ---------------------------------------------------------------------
  '              Starten nchste von drei erlaubten Runden
  ' ----------------------------------------------------------------------------------
  Public Function NachsteRunde() As Boolean
    Dim i%
    SpielRunde += 1
    lblRunde.Text = SpielRunde
    If SpielRunde > 1 Then   ' ------ Karten einsammeln -------------
      For i = 10 To 0 Step -1
        Do Until AusLage.CardCount(i) = 0
          Talon.AddCard(AusLage.RemoveCard(i))
        Loop
      Next i
    End If
    ' ------- Auslegen Zeile 1 -----------------------------------------
    i = 0
    Do Until (i > 10 Or Talon.CardCount = 0)
      AusLage.AddCard(Talon.RemoveCard(), i)
      i += 1
    Loop
    BeimAuslegen = True
    If SpielRunde = 3 Then Return False Else Return True
  End Function

  ' NachsteZeile ---------------------------------------------------------------------
  '              Auslegen der nchsten Zeile von (verdeckten) Talon, soeweit Karten da
  ' ----------------------------------------------------------------------------------
  Public Function NachsteZeile() As Boolean
    Dim tc As Card, i%
    ' -------------- Nchste Zeile -----------------------------------
    i = 0
    Do Until (i > 10 Or (Talon.CardCount = 0))
      tc = AusLage.TopMostCard(i)
      If Not IsNothing(tc) Then tc.Disabled = True
      AusLage.AddCard(Talon.RemoveCard(), i)
      If i > 0 And i < 10 Then AusLage.TopMostCard(i).Disabled = True
      i += 1
    Loop
    ' ---------- Disabled = False, wenn letzte Zeile -----------------
    If Talon.CardCount = 0 Then
      For i = 0 To 10
        AusLage.TopMostCard(i).Disabled = False
      Next i
      BeimAuslegen = False
      Return False
    End If
    BeimAuslegen = True
    Return True
  End Function

End Class
