' --------------------------------------------------------------------------------------
'
' Ablage.VB                                                       Ulrich Mller 19.09.02
'                                                                 PCS1700   VB.NET Final
'
' Klassen zur Erstellung von Patiencen auf Basis von QCard32.DLL (QCard32Wrapper)
' Klassen fr die Ablage
'
' --------------------------------------------------------------------------------------

' Ablage48 ------------------------------------------------------------------------------
'Ablage mit vier/acht Ablagepltze in Reihe mit Mittellcke (bei acht)
' --------------------------------------------------------------------------------------

Public Class Ablage48

  Private qc As QCard32Pool
  Private actCard%(7), NrCols%, NrEmpty%
  Private Cards(7) As Card
  Private XBasis, YBasis, XBreite%, CWidth%
  Private i, j As Integer
  Private FCount%

  ' New --------------------------------------------------------------------------------
  '     Konstruktor
  ' ------------------------------------------------------------------------------------
  Sub New(ByVal QCard As QCard32Pool, ByVal NrCols%, ByVal CardWidth%, _
      ByVal Left%, ByVal Top%, Optional ByVal NrEmpty% = 0)
    qc = QCard
    Me.NrCols = NrCols
    XBasis = Left
    YBasis = Top
    CWidth = CardWidth
    Me.NrEmpty = NrEmpty
    XBreite = CWidth * (NrCols + NrEmpty) - (CWidth - qc.CardWidth)
    Clear()
  End Sub

  ' ColToX ------------------------------------------------------------------------------
  '        Umrechnung Spalten in PixelKoordinate
  ' -------------------------------------------------------------------------------------
  Private Function ColToX(ByVal Col%) As Integer
    Return IIf(Col > 3, Col + NrEmpty, Col) * CWidth + XBasis
  End Function

  ' Clear -------------------------------------------------------------------------------
  '       Lschen der Ablage
  ' -------------------------------------------------------------------------------------
  Public Sub Clear()
    For i = 0 To 7
      actCard(i) = -1
    Next
    FCount = 0
    Paint()
  End Sub

  ' CardCount ---------------------------------------------------------------------------
  '           Gesamtanzahl Karten in der Ablage
  ' -------------------------------------------------------------------------------------
  Public ReadOnly Property CardCount() As Integer
    Get
      Return FCount
    End Get
  End Property

  ' Left / Top -------------------------------------------------------------------------
  '                   Lesen der Koordinaten der linken oberen Ecke
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property Left() As Integer
    Get
      Return XBasis
    End Get
  End Property
  Public ReadOnly Property Top() As Integer
    Get
      Return YBasis
    End Get
  End Property

  ' Range ------------------------------------------------------------------------------
  '       Lesen der Bereichswerte von Ablage48
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property Range() As Rectangle
    Get
      Return New Rectangle(XBasis, YBasis, XBreite, qc.CardHeight)
    End Get
  End Property

  ' XToCol -----------------------------------------------------------------------------
  '        Bestimmung der zu X gehrenden Spalte
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property XToCol(ByVal X%) As Integer
    Get
      Dim B% = Int(X - XBasis)
      If B > (NrCols / 2 * CWidth) Then B -= NrEmpty * CWidth
      Return Int(B / CWidth)
    End Get
  End Property

  ' TopMostCard ------------------------------------------------------------------------
  '             Lesen der obersten Karte auf der angegebenen Spalte, nicht entfernen
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property TopMostCard(ByVal SrcCol%) As Card
    Get
      If actCard(SrcCol) < 0 Then Return Nothing Else Return Cards(SrcCol)
    End Get
  End Property

  ' AddCard ---------------------------------------------------------------------------
  '         Anlegen von SrcCard auf die angegebene Spalte
  ' -----------------------------------------------------------------------------------
  Public Sub AddCard(ByVal SrcCard As Card, ByVal DestCol%)

    actCard(DestCol) += 1
    Cards(DestCol) = SrcCard
    SrcCard.PicX = ColToX(DestCol)
    SrcCard.PicY = YBasis
    SrcCard.Area = AreaCode.Ablage
    SrcCard.PosC = DestCol
    SrcCard.PosR = 0
    qc.DealCard(SrcCard.CardNr, ColToX(DestCol), YBasis)
    SrcCard.Disabled = True
    FCount += 1

  End Sub

  ' Paint ------------------------------------------------------------------------------
  '       Zeichnen des Kartenbildes
  ' ------------------------------------------------------------------------------------
  Public Sub Paint()

    For i = 0 To NrCols - 1
      If actCard(i) < 0 Then
        qc.DrawSymbol(3, ColToX(i), YBasis)
      Else
        qc.DrawCard(Cards(i).CardNr, Cards(i).PicX, YBasis)
      End If
    Next i

  End Sub

  ' IsAscendingSuit ---------------------------------------------------------------------
  '                 Ist die anzulegende Karte aufsteigend (+1) in gleicher Suit
  '                 Den Leerpltzen werden Farben (Suit) und ein Wert 0 zugewiesen
  ' -------------------------------------------------------------------------------------
  Public Function IsAscendingSuit(ByVal SrcCard As Card, ByVal DestCol%) As Boolean

    Dim VSuit, Value As Integer
    Dim DestCard As Card = TopMostCard(DestCol)

    If IsNothing(DestCard) Then
      VSuit = Choose(DestCol + 1, 1, 4, 3, 2, 1, 4, 3, 2)
      Value = 0
    Else
      VSuit = DestCard.Suit
      Value = DestCard.Value
    End If

    Return IIf((VSuit = SrcCard.Suit) And ((Value + 1) = SrcCard.Value), True, False)

  End Function

  ' IsDescendingSuit --------------------------------------------------------------------
  '                  Ist die anzulegende Karte absteigend (-1) in gleicher Suit
  '                  Den Leerpltzen werden Farben (Suit) und ein Wert 14 zugewiesen
  ' -------------------------------------------------------------------------------------
  Public Function IsDescendingSuit(ByVal SrcCard As Card, ByVal DestCol%) As Boolean

    Dim VSuit, Value As Integer
    Dim DestCard As Card = TopMostCard(DestCol)

    If IsNothing(DestCard) Then
      VSuit = Choose(DestCol + 1, 1, 4, 3, 2, 1, 4, 3, 2)
      Value = 14
    Else
      VSuit = DestCard.Suit
      Value = DestCard.Value
    End If

    Return IIf((VSuit = SrcCard.Suit) And ((Value - 1) = SrcCard.Value), True, False)

  End Function

End Class
