' --------------------------------------------------------------------------------------
'
' Auslage.VB                                                      Ulrich Mller 19.09.02
'                                                                 PCS1700   VB.NET Final
'
' Klassen zur Erstellung von Patiencen auf Basis von QCard32.DLL (QCard32Wrapper)
' Klassen fr die Auslage
'
' --------------------------------------------------------------------------------------

' AuslageV -----------------------------------------------------------------------------
' Auslage in Leitern (max. 13) in Reihen mit max. 20 Karten, Blocking 
'         die Anzahl der Leitern ist whlbar, Anlage in wechselnden Farben, absteigend
' --------------------------------------------------------------------------------------

Public Class AuslageV

  Private qc As QCard32Pool
  Private actCard%(12), NrCols%
  Private Cards(12, 39) As Card
  Private XBasis%, YBasis%, XBreite%, CWidth%, XHoehe%
  Dim i, j As Integer

  ' New -------------------------------------------------------------------------------
  '     Konstruktor
  ' -----------------------------------------------------------------------------------
  Sub New(ByVal QCard As QCard32Pool, ByVal NrCols%, ByVal CardWidth%, _
      ByVal Left%, ByVal Top%, ByVal Height%)
    qc = QCard
    Me.NrCols = NrCols
    XBasis = Left
    YBasis = Top
    CWidth = CardWidth
    XBreite = CWidth * NrCols - (CWidth - qc.CardWidth)
    XHoehe = Height
    Clear()
  End Sub

  ' Clear -----------------------------------------------------------------------------
  '       Lschen der Auslage
  ' -----------------------------------------------------------------------------------
  Public Sub Clear()
    For i = 0 To 12
      actCard(i) = -1
    Next
    Paint()
  End Sub

  ' CardCount --------------------------------------------------------------------------
  '           Anzahl Karten auf der angegebenen Spalte ( 0 = keine)
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property CardCount(ByVal PosC%) As Integer
    Get
      Return actCard(PosC) + 1
    End Get
  End Property

  ' Left / Top -------------------------------------------------------------------------
  '                     Lesen der Koordinaten der linken, oberen Ecke
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property Left() As Integer
    Get
      Return XBasis
    End Get
  End Property
  Public ReadOnly Property Top() As Integer
    Get
      Return YBasis
    End Get
  End Property

  ' Range ------------------------------------------------------------------------------
  '       Lesen der Bereichswerte von AuslageV
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property Range() As Rectangle
    Get
      Return New Rectangle(XBasis, YBasis, XBreite, XHoehe)
    End Get
  End Property

  ' XToCol -----------------------------------------------------------------------------
  '        Bestimmung der zu X gehrenden Spalte
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property XToCol(ByVal X%) As Integer
    Get
      Return Int((X - XBasis) / CWidth)
    End Get
  End Property

  ' TopMostCard ------------------------------------------------------------------------
  '             Auslesen der obersten Karte auf der angegebenen Spalte, nicht entfernen
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property TopMostCard(ByVal SrcCol%) As Card
    Get
      If actCard(SrcCol) < 0 Then Return Nothing Else Return Cards(SrcCol, actCard(SrcCol))
    End Get
  End Property

  ' ActualCard -------------------------------------------------------------------------
  '            Auslesen der Karte auf der angebebenen Position, nicht entfernen
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property ActualCard(ByVal PosC%, ByVal PosR%) As Card
    Get
      If actCard(PosC) < PosR Then Return Nothing Else Return Cards(PosC, PosR)
    End Get
  End Property

  ' AddCard ----------------------------------------------------------------------------
  '         Anlegen von SrcCard auf die angegebenen Spalte
  ' ------------------------------------------------------------------------------------
  Public Sub AddCard(ByVal SrcCard As Card, ByVal DestCol%, _
                     Optional ByVal FaceUp As Boolean = True)
    actCard(DestCol) += 1
    Cards(DestCol, actCard(DestCol)) = SrcCard

    SrcCard.PicX = DestCol * CWidth + XBasis
    ' --- Verdeckte Karte 2 Pixel geschuppt, offene qc.Offset ablegen ----
    If actCard(DestCol) > 0 AndAlso Cards(DestCol, actCard(DestCol) - 1).Face _
    Then SrcCard.PicY = Cards(DestCol, actCard(DestCol) - 1).PicY + qc.Offset _
    Else SrcCard.PicY = actCard(DestCol) * 2 + YBasis + 2

    SrcCard.Area = AreaCode.Auslage
    SrcCard.PosC = DestCol
    SrcCard.PosR = actCard(DestCol)
    SrcCard.Face = FaceUp

    ' --- Block unterliegende Karten -----------------------------------------------
    If actCard(DestCol) > 0 Then Cards(DestCol, actCard(DestCol) - 1).Blocked = True

    qc.DealCard(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY)
    SrcCard.Blocked = False         ' --- unBlock oberste Karte

  End Sub

  ' AddRange --------------------------------------------------------------------------
  '          Anlegen von Karten aus Tabelle
  ' -----------------------------------------------------------------------------------
  Public Sub AddRange(ByVal CardList() As Card, ByVal DestCol%)
    Dim i%
    For i = 0 To UBound(CardList)
      AddCard(CardList(i), DestCol)
    Next i
  End Sub

  ' RemoveCard ------------------------------------------------------------------------
  '            Entfernen der obersten Karte der angegebenen Spalte
  ' -----------------------------------------------------------------------------------
  Public Function RemoveCard(ByVal SrcCol%) As Card

    If actCard(SrcCol) < 0 Then Return Nothing

    qc.RemoveCard(Cards(SrcCol, actCard(SrcCol)).CardNr)
    If actCard(SrcCol) > 0 Then               ' --- neue oberste Karte freigeben
      Cards(SrcCol, actCard(SrcCol) - 1).Disabled = False
      Cards(SrcCol, actCard(SrcCol) - 1).Blocked = False
      Cards(SrcCol, actCard(SrcCol) - 1).Face = True
    End If

    actCard(SrcCol) -= 1
    Return Cards(SrcCol, actCard(SrcCol) + 1)

  End Function

  ' RemoveRange ----------------------------------------------------------------------
  '             Entfernen der obersten angegebenen Karten einer Spalte
  ' ----------------------------------------------------------------------------------
  Public Function RemoveRange(ByVal SrcPos%, ByVal SrcCol%) As Card()
    Dim anzCard% = CardCount(SrcCol) - SrcPos
    Dim cc(anzCard - 1) As Card, i As Integer
    For i = anzCard - 1 To 0 Step -1
      cc(i) = RemoveCard(SrcCol)
    Next i
    Return cc
  End Function

  ' Paint -----------------------------------------------------------------------------
  '       Zeichnen des Kartenbildes
  ' -----------------------------------------------------------------------------------
  Public Sub Paint()
    For i = 0 To NrCols - 1
      For j = 0 To actCard(i)        ' --- FaceUp wird von DrawCard beachtet ------
        qc.DrawCard(Cards(i, j).CardNr, Cards(i, j).PicX, Cards(i, j).PicY)
      Next
    Next
  End Sub

  ' IsDescendingChanging ---------------------------------------------------------------
  '                      Ist die anzulegende Karte absteigend in wechselnder Farbe
  ' ------------------------------------------------------------------------------------
  Public Function IsDescendingChanging(ByVal SrcCard As Card, ByVal DestCol%, _
                        Optional ByVal Start As ValueCode = ValueCode.Alle) As Boolean

    Dim DestCard As Card = TopMostCard(DestCol)

    If IsNothing(DestCard) Then  ' --- In leere Spalte nur Knige ------------
      If Start = ValueCode.Alle Then Return True
      If SrcCard.Value = Start Then Return True Else Return False
    End If
    If DestCard.Value = ValueCode.Ass Then Return False ' --- Auf Ass geht nichts

    Return IIf((DestCard.Color <> SrcCard.Color) And _
      (DestCard.Value = (SrcCard.Value + 1)), True, False)

  End Function

  ' IsUpDownSuit -----------------------------------------------------------------------
  '              Ist die anzulegende Karte ab/aufsteigend in gleicher Farbe
  ' ------------------------------------------------------------------------------------
  Public Function IsUpDownSuit(ByVal SrcCard As Card, ByVal DestCol%) As Boolean

    Dim DestCard As Card = TopMostCard(DestCol)
    If IsNothing(DestCard) Then Return True
    Dim res% = Math.Abs(DestCard.Value - SrcCard.Value)
    Return IIf((DestCard.Suit = SrcCard.Suit) And ((res = 1) Or (res = 12)), True, False)
  End Function

  ' IsEnoughSpace --------------------------------------------------------------
  '               Gengend Platz zum Umlegen ber freie Spalten der Auslage
  '               Die Anzahl der leeren Spalten wird gezhlt und bei jeder
  '               neuen freien Spalte zustzlich akkumuliert
  ' ----------------------------------------------------------------------------
  Public Function IsEnoughSpace(ByVal anzCard%, ByVal PosC%) As Boolean

    Dim i% = 0, j% = 0, c% = 0

    For i = 0 To NrCols - 1
      If actCard(i) < 0 Then
        j += 1
        c += j
      End If
    Next i

    If actCard(PosC) >= 0 Then c += 1 Else c -= 1
    Return IIf(c >= anzCard, True, False)

  End Function

End Class
